#!/usr/bin/env python

"""Tests admin-related functionality"""

import os

import pytest

import python_pachyderm
from tests import util


@util.skip_if_no_enterprise()
def test_enterprise():
    client = python_pachyderm.Client()
    client.activate_enterprise(os.environ["PACH_PYTHON_ENTERPRISE_CODE"])
    assert client.get_enterprise_state().state == python_pachyderm.State.ACTIVE.value
    client.deactivate_enterprise()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: client/version/versionpb/version.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='client/version/versionpb/version.proto',
  package='versionpb',
  syntax='proto3',
  serialized_options=b'Z;github.com/pachyderm/pachyderm/src/client/version/versionpb',
  serialized_pb=b'\n&client/version/versionpb/version.proto\x12\tversionpb\x1a\x1bgoogle/protobuf/empty.proto\"J\n\x07Version\x12\r\n\x05major\x18\x01 \x01(\r\x12\r\n\x05minor\x18\x02 \x01(\r\x12\r\n\x05micro\x18\x03 \x01(\r\x12\x12\n\nadditional\x18\x04 \x01(\t2A\n\x03\x41PI\x12:\n\nGetVersion\x12\x16.google.protobuf.Empty\x1a\x12.versionpb.Version\"\x00\x42=Z;github.com/pachyderm/pachyderm/src/client/version/versionpbb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,])




_VERSION = _descriptor.Descriptor(
  name='Version',
  full_name='versionpb.Version',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='major', full_name='versionpb.Version.major', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='minor', full_name='versionpb.Version.minor', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='micro', full_name='versionpb.Version.micro', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='additional', full_name='versionpb.Version.additional', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=82,
  serialized_end=156,
)

DESCRIPTOR.message_types_by_name['Version'] = _VERSION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Version = _reflection.GeneratedProtocolMessageType('Version', (_message.Message,), {
  'DESCRIPTOR' : _VERSION,
  '__module__' : 'client.version.versionpb.version_pb2'
  # @@protoc_insertion_point(class_scope:versionpb.Version)
  })
_sym_db.RegisterMessage(Version)


DESCRIPTOR._options = None

_API = _descriptor.ServiceDescriptor(
  name='API',
  full_name='versionpb.API',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=158,
  serialized_end=223,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetVersion',
    full_name='versionpb.API.GetVersion',
    index=0,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_VERSION,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_API)

DESCRIPTOR.services_by_name['API'] = _API

# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python

"""Tests admin-related functionality"""

import os
from contextlib import contextmanager
from time import sleep

import pytest

import python_pachyderm
from tests import util


@contextmanager
def sandbox():
    client = python_pachyderm.Client()
    client.activate_enterprise(os.environ["PACH_PYTHON_ENTERPRISE_CODE"])
    root_auth_token = None

    try:
        root_auth_token = client.activate_auth("robot:root")
        client.auth_token = root_auth_token
        try:
            yield client
        finally:
            try:
                client.deactivate_auth()
                client.auth_token = None
            except:
                print(
                    "an exception occurred trying to deactivate auth, please manually disable auth with the root auth token: {}".format(
                        root_auth_token
                    )
                )
                raise
    finally:
        client.deactivate_enterprise()


@util.skip_if_no_enterprise()
def test_auth_configuration():
    with sandbox() as client:
        config = client.get_auth_configuration()
        client.set_auth_configuration(config)


@util.skip_if_no_enterprise()
def test_admins():
    with sandbox() as client:
        users = client.get_admins()
        assert users == ["robot:root"]
        client.modify_admins(add=["robot:someuser"])

        # Retry this check three times, in case admin cache is slow
        expected = set(["robot:root", "robot:someuser"])
        for i in range(3):
            users = client.get_admins()
            if set(users) == expected:
                return  # success
            sleep(3)
        print("expected admins {} but got {}".format(expected, set(users)))
        raise


@util.skip_if_no_enterprise()
def test_one_time_password():
    with sandbox() as client:
        otp = client.get_one_time_password()
        client.authenticate_one_time_password(otp)


@util.skip_if_no_enterprise()
def test_authorize():
    with sandbox() as client:
        assert client.authorize("foobar", python_pachyderm.Scope.READER.value)


@util.skip_if_no_enterprise()
def test_who_am_i():
    with sandbox() as client:
        i = client.who_am_i()
        assert i.username == "robot:root"
        assert i.is_admin


@util.skip_if_no_enterprise()
def test_scope():
    with sandbox() as client:
        repo = util.create_test_repo(client, "test_scope")
        scopes = client.get_scope("robot:root", repo)
        assert all(s == python_pachyderm.Scope.NONE.value for s in scopes)
        client.set_scope("robot:root", repo, python_pachyderm.Scope.READER.value)
        scopes = client.get_scope("robot:root", repo)
        assert all(s == python_pachyderm.Scope.NONE.value for s in scopes)


@util.skip_if_no_enterprise()
def test_acl():
    def verify_acl(client, repo):
        acl = client.get_acl(repo)
        assert len(acl.entries) == 1
        assert len(acl.robot_entries) == 0
        assert acl.entries[0].username == "robot:root"
        assert acl.entries[0].scope == python_pachyderm.Scope.OWNER.value
        return acl

    with sandbox() as client:
        repo = util.create_test_repo(client, "test_acl")
        acl = verify_acl(client, repo)
        client.set_acl(repo, acl.entries)
        verify_acl(client, repo)


@util.skip_if_no_enterprise()
def test_auth_token():
    with sandbox() as client:
        auth_token = client.get_auth_token("robot:root", ttl=30)
        assert auth_token.subject == "robot:root"
        client.extend_auth_token(auth_token.token, 60)
        client.revoke_auth_token(auth_token.token)
        with pytest.raises(python_pachyderm.RpcError):
            client.extend_auth_token(auth_token.token, 60)


@util.skip_if_no_enterprise()
def test_groups():
    with sandbox() as client:
        assert client.get_groups() == []
        client.set_groups_for_user("robot:root", ["foogroup"])
        assert client.get_groups() == ["foogroup"]
        assert client.get_users("foogroup") == ["robot:root"]
        client.modify_members("foogroup", remove=["robot:root"])
        assert client.get_groups() == []
        assert client.get_users("foogroup") == []



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: client/health/health.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='client/health/health.proto',
  package='health',
  syntax='proto3',
  serialized_options=b'Z0github.com/pachyderm/pachyderm/src/client/health',
  serialized_pb=b'\n\x1a\x63lient/health/health.proto\x12\x06health\x1a\x1bgoogle/protobuf/empty.proto2D\n\x06Health\x12:\n\x06Health\x12\x16.google.protobuf.Empty\x1a\x16.google.protobuf.Empty\"\x00\x42\x32Z0github.com/pachyderm/pachyderm/src/client/healthb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,])



_sym_db.RegisterFileDescriptor(DESCRIPTOR)


DESCRIPTOR._options = None

_HEALTH = _descriptor.ServiceDescriptor(
  name='Health',
  full_name='health.Health',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=67,
  serialized_end=135,
  methods=[
  _descriptor.MethodDescriptor(
    name='Health',
    full_name='health.Health.Health',
    index=0,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_HEALTH)

DESCRIPTOR.services_by_name['Health'] = _HEALTH

# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python

"""Tests PFS-related functionality"""

import pytest

import python_pachyderm
from tests import util


@util.skip_if_below_pachyderm_version(1, 9, 10)
def test_batch_transaction():
    client = python_pachyderm.Client()
    expected_repo_count = len(client.list_repo()) + 3

    def create_repo_request():
        return python_pachyderm.TransactionRequest(
            create_repo=python_pachyderm.CreateRepoRequest(
                repo=python_pachyderm.Repo(
                    name=util.test_repo_name("test_batch_transaction")
                )
            )
        )

    transaction = client.batch_transaction(
        [
            create_repo_request(),
            create_repo_request(),
            create_repo_request(),
        ]
    )

    assert len(client.list_transaction()) == 0
    assert len(client.list_repo()) == expected_repo_count


@util.skip_if_below_pachyderm_version(1, 9, 0)
def test_transaction_context_mgr():
    client = python_pachyderm.Client()
    expected_repo_count = len(client.list_repo()) + 2

    with client.transaction() as transaction:
        util.create_test_repo(client, "test_transaction_context_mgr")
        util.create_test_repo(client, "test_transaction_context_mgr")

        transactions = client.list_transaction()
        assert len(transactions) == 1
        assert transactions[0].transaction.id == transaction.id
        assert client.inspect_transaction(transaction).transaction.id == transaction.id
        assert (
            client.inspect_transaction(transaction.id).transaction.id == transaction.id
        )

    assert len(client.list_transaction()) == 0
    assert len(client.list_repo()) == expected_repo_count


@util.skip_if_below_pachyderm_version(1, 9, 0)
def test_transaction_context_mgr_nested():
    client = python_pachyderm.Client()

    with client.transaction() as transaction:
        assert client.transaction_id is not None
        old_transaction_id = client.transaction_id

        with client.transaction() as transaction:
            assert client.transaction_id is not None
            assert client.transaction_id != old_transaction_id

        assert client.transaction_id == old_transaction_id


@util.skip_if_below_pachyderm_version(1, 9, 0)
def test_transaction_context_mgr_exception():
    client = python_pachyderm.Client()
    expected_repo_count = len(client.list_repo())

    with pytest.raises(Exception):
        with client.transaction() as transaction:
            util.create_test_repo(client, "test_transaction_context_mgr_exception")
            util.create_test_repo(client, "test_transaction_context_mgr_exception")
            raise Exception("oops!")

    assert len(client.list_transaction()) == 0
    assert len(client.list_repo()) == expected_repo_count


@util.skip_if_below_pachyderm_version(1, 9, 0)
def test_delete_transaction():
    client = python_pachyderm.Client()
    expected_repo_count = len(client.list_repo())

    transaction = client.start_transaction()
    util.create_test_repo(client, "test_delete_transaction")
    util.create_test_repo(client, "test_delete_transaction")
    client.delete_transaction(transaction)

    assert len(client.list_transaction()) == 0
    # even though the transaction was deleted, the repos were still created,
    # because the transaction wasn't tied to the client
    assert len(client.list_repo()) == expected_repo_count + 2

    with pytest.raises(python_pachyderm.RpcError):
        # re-deleting should cause an error
        client.delete_transaction(transaction)


@util.skip_if_below_pachyderm_version(1, 9, 0)
def test_delete_all_transactions():
    client = python_pachyderm.Client()
    client.start_transaction()
    client.start_transaction()
    assert len(client.list_transaction()) == 2
    client.delete_all_transactions()
    assert len(client.list_transaction()) == 0



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from python_pachyderm.proto.transaction import transaction_pb2 as client_dot_transaction_dot_transaction__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2


class APIStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.BatchTransaction = channel.unary_unary(
        '/transaction.API/BatchTransaction',
        request_serializer=client_dot_transaction_dot_transaction__pb2.BatchTransactionRequest.SerializeToString,
        response_deserializer=client_dot_transaction_dot_transaction__pb2.TransactionInfo.FromString,
        )
    self.StartTransaction = channel.unary_unary(
        '/transaction.API/StartTransaction',
        request_serializer=client_dot_transaction_dot_transaction__pb2.StartTransactionRequest.SerializeToString,
        response_deserializer=client_dot_transaction_dot_transaction__pb2.Transaction.FromString,
        )
    self.InspectTransaction = channel.unary_unary(
        '/transaction.API/InspectTransaction',
        request_serializer=client_dot_transaction_dot_transaction__pb2.InspectTransactionRequest.SerializeToString,
        response_deserializer=client_dot_transaction_dot_transaction__pb2.TransactionInfo.FromString,
        )
    self.DeleteTransaction = channel.unary_unary(
        '/transaction.API/DeleteTransaction',
        request_serializer=client_dot_transaction_dot_transaction__pb2.DeleteTransactionRequest.SerializeToString,
        response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )
    self.ListTransaction = channel.unary_unary(
        '/transaction.API/ListTransaction',
        request_serializer=client_dot_transaction_dot_transaction__pb2.ListTransactionRequest.SerializeToString,
        response_deserializer=client_dot_transaction_dot_transaction__pb2.TransactionInfos.FromString,
        )
    self.FinishTransaction = channel.unary_unary(
        '/transaction.API/FinishTransaction',
        request_serializer=client_dot_transaction_dot_transaction__pb2.FinishTransactionRequest.SerializeToString,
        response_deserializer=client_dot_transaction_dot_transaction__pb2.TransactionInfo.FromString,
        )
    self.DeleteAll = channel.unary_unary(
        '/transaction.API/DeleteAll',
        request_serializer=client_dot_transaction_dot_transaction__pb2.DeleteAllRequest.SerializeToString,
        response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
        )


class APIServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def BatchTransaction(self, request, context):
    """Transaction rpcs
    """
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def StartTransaction(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def InspectTransaction(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeleteTransaction(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ListTransaction(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def FinishTransaction(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def DeleteAll(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_APIServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'BatchTransaction': grpc.unary_unary_rpc_method_handler(
          servicer.BatchTransaction,
          request_deserializer=client_dot_transaction_dot_transaction__pb2.BatchTransactionRequest.FromString,
          response_serializer=client_dot_transaction_dot_transaction__pb2.TransactionInfo.SerializeToString,
      ),
      'StartTransaction': grpc.unary_unary_rpc_method_handler(
          servicer.StartTransaction,
          request_deserializer=client_dot_transaction_dot_transaction__pb2.StartTransactionRequest.FromString,
          response_serializer=client_dot_transaction_dot_transaction__pb2.Transaction.SerializeToString,
      ),
      'InspectTransaction': grpc.unary_unary_rpc_method_handler(
          servicer.InspectTransaction,
          request_deserializer=client_dot_transaction_dot_transaction__pb2.InspectTransactionRequest.FromString,
          response_serializer=client_dot_transaction_dot_transaction__pb2.TransactionInfo.SerializeToString,
      ),
      'DeleteTransaction': grpc.unary_unary_rpc_method_handler(
          servicer.DeleteTransaction,
          request_deserializer=client_dot_transaction_dot_transaction__pb2.DeleteTransactionRequest.FromString,
          response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
      ),
      'ListTransaction': grpc.unary_unary_rpc_method_handler(
          servicer.ListTransaction,
          request_deserializer=client_dot_transaction_dot_transaction__pb2.ListTransactionRequest.FromString,
          response_serializer=client_dot_transaction_dot_transaction__pb2.TransactionInfos.SerializeToString,
      ),
      'FinishTransaction': grpc.unary_unary_rpc_method_handler(
          servicer.FinishTransaction,
          request_deserializer=client_dot_transaction_dot_transaction__pb2.FinishTransactionRequest.FromString,
          response_serializer=client_dot_transaction_dot_transaction__pb2.TransactionInfo.SerializeToString,
      ),
      'DeleteAll': grpc.unary_unary_rpc_method_handler(
          servicer.DeleteAll,
          request_deserializer=client_dot_transaction_dot_transaction__pb2.DeleteAllRequest.FromString,
          response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'transaction.API', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
